﻿using System;
using System.Collections.Generic;
using System.IO;
using Ueberpruefung;

namespace Testverwaltung
{
    internal class ExtraTest : Test
    {
        private int currentTestNumber;
        public ExtraTest() : base()
        {
            currentTestNumber = 0;
        }
        public override bool run()
        {
            resultString = "Testing Extras (Function Tree, Comments)\r\n\r\n";
            string pathPrefix = "Tests\\extra\\";
            singleTest(pathPrefix+"FTree\\");
            singleTest(pathPrefix+"Comment\\");
            NonDeadSemErrsTest(pathPrefix + "NonDeadSemErrs\\");
            numTests = currentTestNumber;
            return passed;
        }

        private void singleTest(string path)
        {
            ein.Eingabe_Interface("ADT", path + "Bsp.adt");
            ein.Eingabe_Interface("Task", path + "Bsp.task");
            ein.Eingabe_Interface("Proof", path + "Bsp.proof");
            Ueberpruefer.ueberpruefe(ein.GetContainer());
            resultString += assert(currentTestNumber++, aus.NoErrors(), aus.printAllErrors(ein.GetContainer()));
        }

        /// <summary>
        /// Tests Exceptions of non dead semantic errors
        /// </summary>
        /// <param name="path"></param>
        private void NonDeadSemErrsTest(string path)
        {
            List<string> adtFileNames = new List<string>() { "tree1","tree2","tree3","tree4","tree5","sort","op","ax","chaos" };
            List<string> adtShallValues = new List<string>() 
            {
              "Errors:\r\nError in 9:11: The Operation_Tree plus was inputted with 1 children, expected 2 children!\r\nError in 10:11: The Operation_Tree plus was inputted with 3 children, expected 2 children!\r\n",
              "Error in 10:18: The Tree ink could not be found in Operations of the ADT!",
              "Errors:\r\nError in 9:18: The Tree zero could not be found in Vars or Operations of the ADT!\r\nError in 9:26: The Tree k could not be found in Vars or Operations of the ADT!\r\nError in 9:7: One or more sides of one Equation are null!\r\nError in 10:22: The Tree k could not be found in Vars or Operations of the ADT!\r\n",
              "Errors:\r\nError in 9:11: Error while building the tree plus: The the child (Variable) k with the Sort Nat2 is not Nat!\r\nError in 9:7: The Equation has not the same Sort on its 2 sides! Sort of leftTree: Nat Sort of rightTree: Nat2\r\nError in 11:7: The Equation has not the same Sort on its 2 sides! Sort of leftTree: Nat2 Sort of rightTree: Nat\r\nError in 12:27: Error while building the tree plus: The the child (ComposedTree) mal with the Sort Nat2 is not Nat!\r\nError in 12:7: The Equation has not the same Sort on its 2 sides! Sort of leftTree: Nat2 Sort of rightTree: Nat\r\n",
              "Errors:\r\nError in 8:23: The Variable n was defined more than 1 times.!\r\n",
              "Errors:\r\nError in 4:11: The Sort Nat was defined more than 1 times in Sorts of ADT!\r\nError in 5:0: The sort Nat has building constructors, but no basic constructors.\r\n",
              "Errors:\r\nError in 7:24: Found a Sort Nat2 in Operations that was not defined in Sorts!\r\nError in 7:11: The Operation null was defined more than 1 times in Operations of ADT!\r\n",
              "Errors:\r\nError in 9:7: The Axiom p0 was defined more than 1 times in Axioms of ADT!\r\n",              
              "Errors:\r\nError in 4:11: The Sort Nat was defined more than 1 times in Sorts of ADT!\r\nError in 5:18: Found a Sort Blub in Operations that was not defined in Sorts!\r\nError in 8:17: Found a Sort Nat1 in Operations that was not defined in Sorts!\r\nError in 8:25: Found a Sort Nat2 in Operations that was not defined in Sorts!\r\nError in 8:33: Found a Sort Blub in Operations that was not defined in Sorts!\r\nError in 9:11: The Operation plus was defined more than 1 times in Operations of ADT!\r\nError in 10:17: Found a Sort Blub in Operations that was not defined in Sorts!\r\nError in 10:25: Found a Sort Nat3 in Operations that was not defined in Sorts!\r\nError in 10:33: Found a Sort Nat4 in Operations that was not defined in Sorts!\r\nError in 13:5: Found a Sort Bla in Variable that was not defined in Sorts!\r\nError in 13:14: Found a Sort Nat5 in Variable that was not defined in Sorts!\r\nError in 13:24: Found a Sort Bla2 in Variable that was not defined in Sorts!\r\nError in 13:24: The Variable n was defined more than 1 times.!\r\nError in 16:18: The Tree null could not be found in Vars or Operations of the ADT!\r\nError in 16:7: One or more sides of one Equation are null!\r\nError in 17:7: One or more sides of one Equation are null!\r\nError in 18:17: The Tree null could not be found in Vars or Operations of the ADT!\r\nError in 18:25: The Tree null could not be found in Vars or Operations of the ADT!\r\nError in 18:7: One or more sides of one Equation are null!\r\nError in 19:7: One or more sides of one Equation are null!\r\n"
            };
            for (int i = 0; i < adtFileNames.Count; i++)
                ExceptionTesting("ADT", path + adtFileNames[i]+".adt", adtShallValues[i]);
        }

        private void ExceptionTesting(string inputType, string inputPath, string shallMessage) 
        {
            string ExceptionMessage = "";
            try
            {
                ein.Eingabe_Interface(inputType, inputPath);
            }
            catch (System.Exception e)
            {
                ExceptionMessage = e.Message;
            }
            resultString += assert(currentTestNumber++, shallMessage, ExceptionMessage);
        }
    }
}
